@@ -70,7 +70,7 @@ from zope.schema import ValidationError, getFieldsInOrder
 from zope.schema.interfaces import (
     ConstraintNotSatisfied, IBytes, IField, IObject, RequiredMissing)
 from zope.security.interfaces import Unauthorized
-from zope.security.proxy import removeSecurityProxy
+from zope.security.proxy import getChecker, removeSecurityProxy
 from zope.security.management import checkPermission
 from zope.traversing.browser import absoluteURL, AbsoluteURL
 from zope.traversing.browser.interfaces import IAbsoluteURL
@@ -84,7 +84,7 @@ from lazr.lifecycle.snapshot import Snapshot
 from lazr.restful.interfaces import (
     ICollection, ICollectionField, ICollectionResource, IEntry, IEntryField,
     IEntryFieldResource, IEntryResource, IFieldHTMLRenderer, IFieldMarshaller,
-    IHTTPResource, IJSONPublishable, IReferenceChoice,
+    IHTTPResource, IJSONPublishable, IReferenceChoice, IRepresentationCache,
     IResourceDELETEOperation, IResourceGETOperation, IResourcePOSTOperation,
     IScopedCollection, IServiceRootResource, ITopLevelEntryLink,
     IUnmarshallingDoesntNeedValue, IWebServiceClientRequest,
@@ -97,7 +97,8 @@ from lazr.restful.utils import get_current_web_service_request
 WADL_SCHEMA_FILE = os.path.join(os.path.dirname(__file__),
                                 'wadl20061109.xsd')
 
-# Levels of detail to use when unmarshalling the data.
+# Constants and levels of detail to use when unmarshalling the data.
+MISSING = object()
 NORMAL_DETAIL = object()
 CLOSEUP_DETAIL = object()
 
@@ -599,7 +600,7 @@ class BatchingResourceMixin:
     def batch(self, entries, request):
         """Prepare a batch from a (possibly huge) list of entries.
 
-        :return: A hash:
+        :return: A JSON string representing a hash:
         'entries' contains a list of EntryResource objects for the
           entries that actually made it into this batch
         'total_size' contains the total size of the list.
@@ -608,6 +609,11 @@ class BatchingResourceMixin:
         'prev_url', if present, contains a URL to get the previous batch
          in the list.
         'start' contains the starting index of this batch
+
+        Note that the JSON string will be missing its final curly
+        brace. This is in case the caller wants to add some additional
+        keys to the JSON hash. It's the caller's responsibility to add
+        a '}' to the end of the string returned from this method.
         """
         if not hasattr(entries, '__len__'):
             entries = IFiniteSequence(entries)
@@ -617,8 +623,7 @@ class BatchingResourceMixin:
         resources = [EntryResource(entry, request)
                      for entry in navigator.batch
                      if checkPermission(view_permission, entry)]
-        batch = { 'entries' : resources,
-                  'total_size' : navigator.batch.listlength,
+        batch = { 'total_size' : navigator.batch.listlength,
                   'start' : navigator.batch.start }
         if navigator.batch.start < 0:
             batch['start'] = None
@@ -628,7 +633,17 @@ class BatchingResourceMixin:
         prev_url = navigator.prevBatchURL()
         if prev_url != "":
             batch['prev_collection_link'] = prev_url
-        return batch
+        json_string = simplejson.dumps(batch, cls=ResourceJSONEncoder)
+
+        # String together a bunch of entry representations, possibly
+        # obtained from a representation cache.
+        entry_strings = [
+            resource._representation(HTTPResource.JSON_TYPE)
+            for resource in resources]
+        json_string = (json_string[:-1] + ', "entries": ['
+                       + (", ".join(entry_strings) + ']'))
+        # The caller is responsible for tacking on the final curly brace.
+        return json_string
 
 
 class CustomOperationResourceMixin:
@@ -708,8 +723,6 @@ class CustomOperationResourceMixin:
             return "DELETE not supported."
         return operation()
 
-        return operation()
-
 
 class FieldUnmarshallerMixin:
 
@@ -733,12 +746,11 @@ class FieldUnmarshallerMixin:
 
         :return: a 2-tuple (representation_name, representation_value).
         """
-        missing = object()
-        cached_value = missing
+        cached_value = MISSING
         if detail is NORMAL_DETAIL:
             cached_value = self._unmarshalled_field_cache.get(
-                field_name, missing)
-        if cached_value is not missing:
+                field_name, MISSING)
+        if cached_value is not MISSING:
             return cached_value
 
         field = field.bind(self.context)
@@ -1442,6 +1454,29 @@ class EntryResource(CustomOperationResourceMixin,
                     self.request), self.request),
             adapter.singular_type)
 
+    @property
+    def redacted_fields(self):
+        """Names the fields the current user doesn't have permission to see."""
+        failures = []
+        checker = getChecker(self.context)
+        for name, field in getFieldsInOrder(self.entry.schema):
+            try:
+                # Can we view the field's value? We check the
+                # permission directly using the Zope permission
+                # checker, because doing it indirectly by fetching the
+                # value may have very slow side effects such as
+                # database hits.
+                tagged_values = field.getTaggedValue('lazr.restful.exported')
+                original_name = tagged_values['original_name']
+                checker.check(self.context, original_name)
+            except Unauthorized:
+                # This is an expensive operation that will make this
+                # request more expensive still, but it happens
+                # relatively rarely.
+                repr_name, repr_value = self._unmarshallField(name, field)
+                failures.append(repr_name)
+        return failures
+
     def isModifiableField(self, field, is_external_client):
         """Returns true if this field's value can be changed.
 
@@ -1463,10 +1498,45 @@ class EntryResource(CustomOperationResourceMixin,
 
     def _representation(self, media_type):
         """Return a representation of this entry, of the given media type."""
+
         if media_type in [self.WADL_TYPE, self.DEPRECATED_WADL_TYPE]:
             return self.toWADL().encode("utf-8")
         elif media_type == self.JSON_TYPE:
-            return simplejson.dumps(self, cls=ResourceJSONEncoder)
+            cache = None
+            try:
+                cache = getUtility(IRepresentationCache)
+                representation = cache.get(
+                    self.context, self.JSON_TYPE, self.request.version)
+            except ComponentLookupError:
+                # There's no representation cache.
+                representation = None
+
+            redacted_fields = self.redacted_fields
+            if representation is None:
+                # Either there is no cache, or the representation
+                # wasn't in the cache.
+                representation = simplejson.dumps(self, cls=ResourceJSONEncoder)
+                # If there's a cache, and this representation doesn't
+                # contain any redactions, store it in the cache.
+                if cache is not None and len(redacted_fields) == 0:
+                    cache.set(self.context, self.JSON_TYPE,
+                              self.request.version, representation)
+            else:
+                # We have a representation, but we might not be able
+                # to use it as-is.
+                if len(redacted_fields) != 0:
+                    # We can't use the representation as is. We need
+                    # to deserialize it, redact certain fields, and
+                    # reserialize it. Hopefully this is faster than
+                    # generating the representation from scratch!
+                    json = simplejson.loads(representation)
+                    for field in redacted_fields:
+                        json[field] = self.REDACTED_VALUE
+                    # There's no need to use the ResourceJSONEncoder,
+                    # because we loaded the cached representation
+                    # using the standard decoder.
+                    representation = simplejson.dumps(json)
+            return representation
         elif media_type == self.XHTML_TYPE:
             return self.toXHTML().encode("utf-8")
         else:
@@ -1516,7 +1586,7 @@ class CollectionResource(BatchingResourceMixin,
             result = self.batch(entries)
 
         self.request.response.setHeader('Content-type', self.JSON_TYPE)
-        return simplejson.dumps(result, cls=ResourceJSONEncoder)
+        return result
 
     def batch(self, entries=None):
         """Return a JSON representation of a batch of entries.
@@ -1526,7 +1596,9 @@ class CollectionResource(BatchingResourceMixin,
         if entries is None:
             entries = self.collection.find()
         result = super(CollectionResource, self).batch(entries, self.request)
-        result['resource_type_link'] = self.type_url
+        result += (
+            ', "resource_type_link" : ' + simplejson.dumps(self.type_url)
+            + '}')
         return result
 
     @property
