@@ -34,6 +34,7 @@ __all__ = [
     'IHTTPResource',
     'IJSONPublishable',
     'IJSONRequestCache',
+    'IRepresentationCache',
     'IResourceOperation',
     'IResourceGETOperation',
     'IResourceDELETEOperation',
@@ -606,4 +607,55 @@ class ITraverseWithGet(IPublishTraverse):
         """Traverse to a sub-object."""
 
 
+class IRepresentationCache(Interface):
+    """A cache for resource representations.
+
+    Register an object as the utility for this interface and
+    lazr.restful will use that object to cache resource
+    representations. If no object is registered as the utility,
+    representations will not be cached.
+
+    This is designed to be used with memcached, but you can plug in
+    other key-value stores. Note that this cache is intended to store
+    string representations, not deserialized JSON objects or anything
+    else.
+    """
+
+    def get(object, media_Type, version, default=None):
+        """Retrieve a representation from the cache.
+
+        :param object: An IEntry--the object whose representation you want.
+        :param media_type: The media type of the representation to get.
+        :param version: The version of the web service for which to
+            fetch a representation.
+        :param default: The object to return if no representation is
+            cached for this object.
+
+        :return: A string representation, or `default`.
+        """
+        pass
+
+    def set(object, media_type, version, representation):
+        """Add a representation to the cache.
+
+        :param object: An IEntry--the object whose representation this is.
+        :param media_type: The media type of the representation.
+        :param version: The version of the web service in which this
+            representation should be stored.
+        :param representation: The string representation to store.
+        """
+        pass
+
+    def delete(object):
+        """Remove *all* of an object's representations from the cache.
+
+        This means representations for every (supported) media type
+        and every version of the web service. Currently the only
+        supported media type is 'application/json'.
+
+        :param object: An IEntry--the object being represented.
+        """
+        pass
+
+
 InvalidBatchSizeError.__lazr_webservice_error__ = 400
